import com.treemap.DefaultTreeMapModel
import com.treemap.RenderingFactory
import com.treemap.TreeMap
import com.treemap.treemap
import kotlinx.browser.document
import kotlinx.browser.window
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.await
import kotlinx.coroutines.launch
import org.mkui.font.MkFont
import org.mkui.labeling.EnhancedLabel
import org.mkui.palette.PaletteFactory
import org.molap.dataframe.JsonDataFrame
import react.*
import react.dom.render

private val demo = functionalComponent<RProps> {
    val (model, setModel) = useState<DefaultTreeMapModel<Int, String?>?>(null)

    useEffectOnce {
        GlobalScope.launch {
            TreeMap.setLicenseKey("My Company", "ABC12-ABC12-ABC12-ABC12-ABC12-ABC12")

            val json = window.fetch("https://public.treemap.com/rest/json/Forbes%20Global%202000%20-%202021.mtm").await().text().await()
            val dataFrame = JsonDataFrame(json)
            val treeMapModel = DefaultTreeMapModel(dataFrame)

            val settings = treeMapModel.settings

            // General
            settings.rendering = RenderingFactory.FLAT

            // Group by
            settings.groupByColumns = listOf("Sector", "Industry")

            // Size
            settings.sizeColumn = "Market Value"

            // Color
            settings.colorColumn = "Profits"
            val profitsSettings = settings.getColumnSettings("Profits")
            val negpos = PaletteFactory.instance["negpos"]!!.getPalette()
            val colorMap = treeMapModel.getColorMap("Profits")
            colorMap!!.palette = negpos
            colorMap.interval!!.setValue(-63.93, 127.86)

            // Label
            val companySettings = settings.getColumnSettings("Company")
            companySettings.setLabelingFont(MkFont("Helvetica", 9.0)) // 9 points is the minimum size that will be displayed
            companySettings.setLabelingMinimumCharactersToDisplay(5)
            companySettings.setLabelingResizeTextToFitShape(true)
            companySettings.setLabelingVerticalAlignment(EnhancedLabel.CENTER)
            companySettings.setLabelingHorizontalAlignment(EnhancedLabel.CENTER)

            setModel(treeMapModel)
        }
    }

    treemap {
        attrs {
            this.model = model
        }
    }
}

fun RBuilder.demo() = child(demo) {}

fun main() {
    println("TreeMap starting up")

    window.onload = {
        render(document.getElementById("root")) {
            demo()
        }
    }
}